var front = null;

/**
 * This class encapsules all functions for the front side of the widget.
 */
function Front() {
	this._init();
}

Front.prototype._init = function() {
	$('#start').click(timer.startOrPauseTimer);
	$('#start').bind('mouseover mouseout mousedown mouseup', 
		 			this.setStartButtonPressed);

	elems = Array('timerTarget', 'hours', 'minutes', 'seconds');
	l = elems.length;
	for (i in elems) {
		i = parseInt(i);
		val = elems[i];
		$('#' + val).bind('click', {editor: val + 'Input'}, this.showEdit);
		$('#' + val + 'Input').bind('blur keypress', {display: val}, 
			this.setText);
		$('#' + val + 'Input').bind('keypress', 
			{pre: elems[(i+l-1) % l], nxt: elems[(i+1) % l]}, this.cycleInputs);
		if (i > 0)
			$('#' + val + 'Input').keypress(this.raiseOrLowerValue);
	}
};
	
/**
 * Handles the onMouseDown and onMouseUp events for the start/pause button.
 *
 * @param event: The js event.
 */
Front.prototype.setStartButtonPressed = function(event) {
	pressed = (event.type == 'mouseover' || event.type == 'mousedown');
	play = timer.timerInterval ? false : true	;
	front.setStartButton(play, pressed);
};

/**;
 * Sets either the play or the pause button to the widget.
 *
 * @param play: Set the play button if true, else the pause button.
 */
Front.prototype.setStartButton = function(play, pressed) {
	img = 'Images/'
	img += play ? 'start' : 'pause';
	img += pressed ? '_clicked' : '';
	img += '.png';
	$('#start').attr('src', img);
};

/**
 * Show the text input field to set an event or the time.
 *
 * @param event: The js event.
 */
Front.prototype.showEdit = function(event) {
	if (timer.timerInterval)
		$('#start').click();
	display = $(event.target);
	editor = $('#' + event.data.editor);
	editor.css('visibility', 'visible');
	display.css('visiblity', 'hidden');
	editor.get(0).focus();
	editor.get(0).select();
};

/**
 * Set the text from the input field to the display container and hide
 * the input field.
 *
 * @param event: The js event.
 */
Front.prototype.setText = function(event) {
	if (event.type == 'keypress' && event.keyCode != 13)
		return;
	editor = $(event.target);
	display = $('#' + event.data.display);
	if (editor.css('visibility') == 'visible') {
		if (timer.timerInterval)
			clearInterval(timerInterval);
		if (editor.val().length < 1) {
			if (editor.attr('id') == 'timerTargetInput')
				editor.val('Irgendwas');
			else
				editor.val('00');
		}
		display.text(editor.attr('id') == 'timerTargetInput'
				? editor.val()
				: front.formatTwoDigits(editor.val()));
		display.css('visibility', 'visible');
		editor.css('visibility', 'hidden');
	}
	// Start the timer if the user pressed enter.
	if (event.keyCode == 13)
		$('#start').click();
};

/**
 * Raise or lower the value in the hours, minutes or seconds field by one if
 * the user presses the up or down key.
 *
 * It seems like the dashboard uses other key codes for up/down as all all
 * other apps do. Weird???
 *
 * @param event: The js event.
 */
Front.prototype.raiseOrLowerValue = function(event) {
	if (event.keyCode != 38 && event.keyCode != 40
			&& event.keyCode != 63232 && event.keyCode != 63233)
		return;

	if (event.keyCode == 63232) event.keyCode = 38;
	if (event.keyCode == 63233) event.keyCode = 40;

	event.preventDefault();
	e = event.target;
	val = parseInt(e.value, 10);
	if (event.keyCode == 38 && val < 59)
		e.value = val + 1;
	else if (event.keyCode == 40 && val > 0)
		e.value = val -1;
};

/**
 * Cycles through the four input elements if the user presses tab or shift-tab.
 *
 * @param event: The js event.
 */
Front.prototype.cycleInputs = function(event) {
	if (event.keyCode != 9)
		return;
	event.preventDefault();
	if (event.shiftKey)
		$('#' + event.data.pre).click();
	else
		$('#' + event.data.nxt).click()
	$(event.target).blur();
};

/**
 * Update the timer display with the currently remaining time.
 */
Front.prototype.updateTimerDisplay = function() {
	$('#hours').text(front.formatTwoDigits(timer.remainingHours));
	$('#minutes').text(front.formatTwoDigits(timer.remainingMinutes));
	$('#seconds').text(front.formatTwoDigits(timer.remainingSeconds));
};

/**
 * Formats a number to to digits with leading zero.
 *
 * @param number: The number to format.
 * @return: The formatted number.
 */
Front.prototype.formatTwoDigits = function(number) {
	number = parseInt(number, 10);
	if (number < 0 || isNaN(number)) number = 0;

	number = number.toString(10);
	if (number.length == 1)
		number = '0' + number;
	return number;
};

/**
 * Resets the display to 00:00:00.
 */
Front.prototype.resetDisplay = function() {
	$('#hours').text('00');
	$('#minutes').text('00');
	$('#seconds').text('00');
};
