var timer = null;

/**
 * This class implements the countdown timer and the alarm function.
 */
function Timer() {
	this._targetTime = null;
	this._isTargetReached = false;
	this.remainingHours = -1;
	this.remainingMinutes = -1;
	this.remainingSeconds = -1;
	this.timerInterval = null;
}

/**
 * This function is beeing executed if the start/pause button is pressed.
 * It either pauses the current countdown or starts a new one with the
 * currently displayed target time.
 */
Timer.prototype.startOrPauseTimer = function(event) {
	if (timer.timerInterval) {
		clearInterval(timer.timerInterval);
		timer.timerInterval = null;
		front.setStartButton(true, false);
	}
	else {
		var h = parseInt($('#hours').text(), 10);
		var m = parseInt($('#minutes').text(), 10);
		var s = parseInt($('#seconds').text(), 10);
		m += h * 60;
		s += m * 60;
		timer._targetTime = Date.parse(new Date()) + s * 1000;
		timer._isTargetReached = false;
		// Check if target time has already been passed.
		if (timer._targetTime > Date.parse(new Date())) {
			front.setStartButton(false, false);
			timer.timerInterval = setInterval('timer.runTimer()', 1000);
		}
		else {
			front.resetDisplay();
			front.setStartButton(true, false);
		}
	}
};

/**
 * The run timer action first calculates the remaining time, then updates
 * the displayed time and finally checks wheter to alarm the user or not.
 */
Timer.prototype.runTimer = function() {
    timer.calculateRemainingTime();
    front.updateTimerDisplay();
    timer.checkForAlarm();
};

/**
 * Calculate the remaining hours, minutes and seconds and write them to their
 * corresponding global vars.
 */
Timer.prototype.calculateRemainingTime = function() {
	currentTime = Date.parse(new Date());
	timer.remainingSeconds = (timer._targetTime - currentTime) / 1000;
	timer.remainingMinutes = parseInt(timer.remainingSeconds / 60);
	timer.remainingHours = parseInt(timer.remainingMinutes / 60);
	timer.remainingSeconds %= 60;
	timer.remainingMinutes %= 60;
};

/**
 * Check if the target time is reach and alarm the user.
 */
Timer.prototype.checkForAlarm = function() {
	if (timer.remainingSeconds <= 0 && timer.remainingMinutes <= 0 
			&& timer.remainingHours <= 0 && !timer._isTargetReached) {
		clearInterval(timer.timerInterval);
		timer.timerInterval = null;
		timer._isTargetReached = true;
		front.resetDisplay();
		front.setStartButton(true, false);
        timer.alarm();
	}
};

/**
 * Alarm the user according to his preferences.
 *
 * If type is not set, all alarms set in alarm will be used. This is the case
 * for normal alarms. Else only the alarm specified via type will be used. This
 * is the case if the user switches is prefrences and a “preview alarm” is
 * raised.
 *
 * @param type: Specifies a special (and only) alarm to be raised. May be
 *              "sound", "voice" or "growl". If not set, all alarm types will
 *              be used.
 */
Timer.prototype.alarm = function(type) {
	var title = getLocalizedString('Tea Timer');
	var message = $('#timerTarget').text() + ' '
			+ getLocalizedString('is ready.');

	if ((!type || type == 'sound') && alarm.sound != 'none') {
		$('#soundPlayer').remove();
		$('#alarm').append('<embed name="soundPlayer" id="soundPlayer" '
				+ 'src="/System/Library/Sounds/' + alarm.sound + '.aiff" '
				+ 'autostart="false" hidden="true" enablejavascript="true" />');
		$('#soundPlayer').get(0).Play();
	}
	if (window.widget && (!type || type == 'growl') && alarm.growl) {
		widget.system('/usr/bin/env python2.5 growl.py "' + title + '" "' 
				+ message + '"', null);
	}
	if (window.widget && (!type || type == 'voice') && alarm.voice != 'none') {
		widget.system('/usr/bin/osascript -e \'say "' + message
						+ '" using "' + alarm.voice + '"\'', null);
	}
};
